<?php
// +-----------------------------------------------------------------------+
// | Plugin Name : Export Image metadata                                   |
// | Plugin Version : 0.1                                                  |
// | Plugin Author: Eric                                                   |
// | Plugin Description :                                                  |
// | Ce module est base sur le module existant pour le telechargement,     |
// | cette version permet le telechargement sur plusieurs pages.           |
// +-----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or modify  |
// | it under the terms of the GNU General Public License as published by  |
// | the Free Software Foundation                                          |
// |                                                                       |
// | This program is distributed in the hope that it will be useful, but   |
// | WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      |
// | General Public License for more details.                              |
// |                                                                       |
// | You should have received a copy of the GNU General Public License     |
// | along with this program; if not, write to the Free Software           |
// | Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, |
// | USA.               '                                                  |
// +-----------------------------------------------------------------------+

class ExportImageMetadata
{
  private $use_exiftool, $use_exiv2, $use_whole_desc, $isMSWin, $useUTF8_txt, $useUTF8_cp, $eol;
  private $exiftoolDefaultCmd, $exiv2DefaultCmd;
  private $isTest = false; //true; 

  function ExportImageMetadata($use_exiftool_p, $use_exiv2_p, $use_whole_desc_p, $useUTF8_txt)
  {
	$this->use_exiftool = $use_exiftool_p;
	$this->use_exiv2 = $use_exiv2_p;
	$this->use_whole_desc = $use_whole_desc_p;
	$this->isMSWin = stristr(php_uname(), 'windows');
	$this->eol = $this->isMSWin ? "\r\n" : "\n";
	$this->useUTF8_txt = $useUTF8_txt; // encoding of the batch file
	$this->useUTF8_cp = false; // pb under windows xp for the author with this code page...
  }

  public function set_tool_pathes($exiftoolPath, $exiv2path)
  {
	if ($this->use_exiftool)
	{
		$this->exiftoolDefaultCmd = $this->init_exif_tool_cmd_name($exiftoolPath, "exiftool");
		if (!$this->useUTF8_txt) 
			$this->exiftoolDefaultCmd .= " -L"; // convert to Latin1 internaly, see FAQ #10 at http://www.sno.phy.queensu.ca/~phil/exiftool/faq.html
		$this->exiftoolDefaultCmd .= " -overwrite_original"; // no backup!
	}
	if ($this->use_exiv2)
	{
		$this->exiv2DefaultCmd = $this->init_exif_tool_cmd_name($exiv2path, "exiv2");
	       	$this->exiv2DefaultCmd .= " -k"; // conserve original date
	}
  }

  private function init_exif_tool_cmd_name($toolPath, $toolName)
  {
	return (file_exists($toolPath) && strpos(strtolower($toolPath), strtolower($toolName)) !== false) ? "\"".realpath($toolPath)."\"" : $toolName;
  }

  public function image_properties_to_file($destPath)
  { 		
	$exiftoolScript = ""; $exiv2Script = "";
	if ($this->use_exiftool)
	{
		$exiftoolScript = $destPath ."/writeTagsToIptcWithExifTool.cmd";
		$fp_exiftool = $this->open_create_file($exiftoolScript);
	}
	if ($this->use_exiv2)
	{
		$exiv2Script = $destPath ."/writeTagsToIptcWithExiv2.cmd";
		$fp_exiv2 = $this->open_create_file($exiv2Script);
	}

	$datas = $this->isTest ? $this->get_image_properties_test() : $this->get_image_properties();
	$nbr_tag = 0;

	while ($row = mysql_fetch_row($datas)) // row to avoid conflict with name
	{		
		$image_path = $row[5];
		
		if ($this->isTest || file_exists($image_path))
		{
			$writeOK = false;
			if ($this->use_exiftool)
			{
				$writeOK = $this->convert_row_to_file_exiftool($row, $fp_exiftool);
			}
			if ($this->use_exiv2)
			{
				$writeOK = $this->convert_row_to_file_exiv2($row, $fp_exiv2);
			}
			if ($writeOK)
			{
				$nbr_tag = $nbr_tag + 1;
			}
		}
	}


	if ($this->use_exiftool && isset($fp_exiftool))
	{
		$this->close_file($nbr_tag, $fp_exiftool);
	}
	if ($this->use_exiv2 && isset($fp_exiv2))
	{
		$this->close_file($nbr_tag, $fp_exiv2);
	}
	return $nbr_tag;
  }

  // IO function
  private function open_create_file($destPath)
  {
	// prepare output file
	if(file_exists($destPath))
	{
		if (!$this->isMSWin) chmod($destPath, 0777);
		unlink($destPath);
	}
	touch($destPath);
	$fp = fopen($destPath,"w"); 
	if ($this->isMSWin && isset($fp) )
	{
		// the default command prompt is stupid and won't display the correct character set, so we need to change the code page
		if ($this->useUTF8_cp)
		{ fwrite($fp, "chcp 65001".$this->eol); } // UTF-8, prevent cmd to work, see perhaps http://blogs.msdn.com/michkap/archive/2006/03/06/544251.aspx
		else
		{ fwrite($fp, "chcp 1252".$this->eol); }  // default Windows Latin1		
	}
	return $fp;
  }

  private function close_file($nbr_tag, $fp)
  {
	if (isset($fp))
	{
		if ($nbr_tag > 0)
		{
			fwrite($fp, $this->eol);
		}
		fclose($fp);
	}
  }

  // prepare data for script
  private function convert_row_to_file_exiv2($row, $fp_exiv2)
  {
	if (!($this->use_exiv2 && isset($fp_exiv2)))
		return false;
	$name =  $this->explode_description($this->decode_html_string_to_unicode($row[0], true));
	$description = $this->explode_description($this->decode_html_string_to_unicode($row[1], true));
	$author = $this->decode_html_string_to_unicode($row[2], true);
	//$date_creation = $this->decode_html_string_to_unicode($row[3], true);
	$tags = $this->decode_html_string_to_unicode($row[4], true);
	$image_path = $row[5];
	
	$exiv2Cmd = $this->exiv2DefaultCmd;

	if (strlen($name) > 0)
	{
		$exiv2Cmd .= " -M\"set Iptc.Application2.ObjectName ".$name."\"";
	}
	if (strlen($description) > 0)
	{							
		$exiv2Cmd .= " -M\"set Iptc.Application2.Caption ".$description."\"";
	}
	if (strlen($author) > 0)
	{
		$exiv2Cmd .= " -M\"set Iptc.Application2.Writer ".$author."\"";
	}	
	if (strlen($tags) > 0)			
	{
		$exiv2Cmd .= " -M\"add Iptc.Application2.Keywords ".$tags."\"";
	}
	$exiv2Cmd .= " ";
       	$exiv2Cmd .= $this->isTest ? $image_path : realpath(PHPWG_ROOT_PATH.$image_path);

	return (fwrite($fp_exiv2, $exiv2Cmd.$this->eol) !== FALSE);
  }

  private function convert_row_to_file_exiftool($row, $fp_exiftool)
  {
	if (!($this->use_exiftool && isset($fp_exiftool)))
		return false;
	$name = $this->explode_description($this->decode_html_string_to_unicode($row[0], false));
	$description = $this->explode_description($this->decode_html_string_to_unicode($row[1], false));
	$author = $this->decode_html_string_to_unicode($row[2], false);
	//$date_creation = $this->decode_html_string_to_unicode($row[3], false);
	$tags = $this->decode_html_string_to_unicode($row[4], false);
	$image_path = $row[5];

	$exiftoolCmd = $this->exiftoolDefaultCmd;	

	if (strlen($name) > 0)
	{
		$exiftoolCmd .= " -IPTC:ObjectName=\"".$name."\"";
	}
	if (strlen($description) > 0)
	{							
		$exiftoolCmd .= " -IPTC:Caption-Abstract=\"".$description."\"";
	}
	if (strlen($author) > 0)
	{
		$exiftoolCmd .= " -IPTC:Writer-Editor=\"".$author."\"";
	}	
	if (strlen($tags) > 0)			
	{
		$exiftoolCmd .= " -IPTC:Keywords=\"".$tags."\"";
	}
	$exiftoolCmd .= " ";
       	$exiftoolCmd .= $this->isTest ? $image_path : realpath(PHPWG_ROOT_PATH.$image_path);	

	return (fwrite($fp_exiftool, $exiftoolCmd.$this->eol) !== FALSE);
  }

    // html function
  private function decode_html_string_to_unicode($string, $forceANSI) 
  {
	if (isset($string) && strlen($string) > 0)
	{
		$txt_encoding = ($this->useUTF8_txt && !$forceANSI) ? 'UTF-8' : 'ISO-8859-15';
		$string = html_entity_decode(trim($string), ENT_QUOTES, $txt_encoding);
		$string = stripslashes($string);
	}
	else $string = "";
	return($string);
  } 

    // remove end of line char from desc and keep if required only the current language
  private function explode_description($description)
  {
	if (!$this->use_whole_desc)
	{
		$description = $this->get_user_language_desc($description);	
	}
	return str_replace( array( '<br>', '<br />', "\n", "\r" ), array('', '', '', ''), $description );
  }
  
  // Traite les balises [lang=xx], copy&paste from ExtendedDescription plugin, main.inc.php
  private function get_user_language_desc($desc)
  {
	global $user;
  
	$user_lang = substr($user['language'], 0, 2);

	if (!substr_count(strtolower($desc), '[lang=' . $user_lang . ']'))
	{
		$user_lang = 'default';
  	}
  
	if (substr_count(strtolower($desc), '[lang=' . $user_lang . ']'))
	{
		// la balise avec la langue de l'utilisateur a été trouvée
		$patterns[] = '#(^|\[/lang\])(.*?)(\[lang=(' . $user_lang . '|all)\]|$)#is';
		$replacements[] = '';
		$patterns[] = '#\[lang=(' . $user_lang . '|all)\](.*?)\[/lang\]#is';
		$replacements[] = '\\1';
	}
	else
	{
		// la balise avec la langue de l'utilisateur n'a pas été trouvée
		// On prend tout ce qui est hors balise
		$patterns[] = '#\[lang=all\](.*?)\[/lang\]#is';
		$replacements[] = '\\1';
		$patterns[] = '#\[lang=.*\].*\[/lang\]#is';
		$replacements[] = '';
	}
	return preg_replace($patterns, $replacements, $desc);
  }

  // data access
  private function get_image_properties() 
  {
	// get direct properties and tags, even if no tags, with either name, comment or tags not null
	$img_query = 'SELECT img.name, img.comment, img.author, img.date_creation, GROUP_CONCAT(tags.name), img.path
		FROM '.IMAGES_TABLE.' AS img
		LEFT OUTER JOIN '.IMAGE_TAG_TABLE.' AS img_tag ON img_tag.image_id = img.id
		LEFT OUTER JOIN '.TAGS_TABLE.' AS tags ON tags.id = img_tag.tag_id
		WHERE ( LENGTH(img.name) > 0 OR LENGTH(img.comment) > 0 OR LENGTH(img.author) > 0 OR LENGTH(tags.name) > 0 )  
		GROUP BY img.id, img.name, img.comment, img.author, img.path
		ORDER BY  img.id ASC; ';
	
	return pwg_query($img_query);
  }

  private function get_image_properties_test() 
  {
	// restritive query with image.ID for my own tests with 2 copied images with file instead of path
	$img_query = 'SELECT img.name, img.comment, img.author, img.date_creation, GROUP_CONCAT(tags.name), img.file
		FROM '.IMAGES_TABLE.' AS img
		LEFT OUTER JOIN '.IMAGE_TAG_TABLE.' AS img_tag ON img_tag.image_id = img.id
		LEFT OUTER JOIN '.TAGS_TABLE.' AS tags ON tags.id = img_tag.tag_id
		WHERE ( ( LENGTH(img.name) > 0 OR LENGTH(img.comment) > 0 OR LENGTH(img.author) > 0 OR LENGTH(tags.name) > 0 ) AND (img.id = 2 OR img.id = 655))
		GROUP BY img.id, img.name, img.comment, img.author, img.file
		ORDER BY  img.id ASC; ';
	
	return pwg_query($img_query);
  }
}
